/*
 * trigger.c for MUDix - Enigma
 */
#include <conf.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <ncurses.h>
#include <panel.h>
#include "mudix.h"


void show_triggers(void)
{
    static int		 current;
    	   TRIGGER 	*trig;
    	   char    	*str;
    	   int      	 line, i, width, height, centre, wsize;

    width  = LEN_COL-4;
    height = 20;
    wsize  = height-2;
    centre = ((width-2)/2)-1;

    wresize(wMsg, height, width);
    werase(wMsg);
    move_panel(pMsg, 1, 2);

    if (panel_hidden(pMsg) == PANEL_HIDDEN)
	current = 0;

    for (line=1, i=0, trig = settings->trigger_list; trig; trig = trig->next) {
    	if (!trig->in || !trig->out)
            continue;

	i+=2;
	if (i < (current*wsize)+2)
	    continue;

      	mvwprintw(wMsg, line++, 1, "%2d%c (%2d): %s",
		(i/2), trig->enabled? '*': ' ', trig->level, trig->in); 
      	mvwprintw(wMsg, line++, 1, "Response: %s", 
		trig->level == TRG_PASSWORD?  "*********": trig->out); 

	if (line >= wsize && trig->next) {
	    current++;
	    break;
	}

	if (!trig->next)	/* last one */
	    current = 0;
    }

    draw_border(wMsg);
    str = "Triggers";
    mvwprintw(wMsg, 0, centre-strlen(str)/2, str);
    show_panel(pMsg);
    return;
}

TRIGGER *new_trigger(int level, bool last)
{
    TRIGGER *trigger;

    if (!(trigger = (TRIGGER *)malloc(sizeof(TRIGGER))))
	return NULL; 
     
    if (last && settings->trigger_list) {
        TRIGGER *iTrig;

        for (iTrig = settings->trigger_list; iTrig; iTrig = iTrig->next) {
            if (!iTrig->next)
                break;
        }
        iTrig->next = trigger;
        trigger->next = NULL;
    }
    else {
    	trigger->next = settings->trigger_list;
    	settings->trigger_list = trigger;
    }

    trigger->in 	   = NULL;
    trigger->out	   = NULL;
    trigger->inp 	   = NULL;
    trigger->pArg 	   = NULL;
    trigger->enabled	   = FALSE;
    trigger->level	   = level;

    return trigger;
}

void free_trigger(TRIGGER *trigger)
{
    TRIGGER *lookup;

    if (!trigger)
	return;

    for (lookup = settings->trigger_list; lookup; lookup = lookup->next) {
        if (lookup->next == trigger) {
            lookup->next = trigger->next;
            break;
        }
    }

    if (trigger == settings->trigger_list)
        settings->trigger_list = trigger->next;

    if (trigger->in)
	free(trigger->in);
    if (trigger->out)
	free(trigger->out);
    trigger->inp = NULL;

    free(trigger);
    return;
}

TRIGGER *trig_lookup(TRIGGER *beg, int level)
{
    TRIGGER *trig;

    for (trig = (beg? beg: settings->trigger_list); trig; trig = trig->next) {
	if (!trig->in || !trig->out)
	    continue;

	if (trig->level == level)
	    break;
    }

    return trig;
}

void trigger_check(char *pBuf)
{
    TRIGGER *trig;
    char    *pTmp;

    for (trig = settings->trigger_list; trig; trig = trig->next) {
        if (!trig->enabled || !trig->in || !trig->out || !trig->inp)
            continue;
	
	pTmp = pBuf;
	while (*pTmp && *(trig->inp)) {
	    if (trig->pArg) {
		if (!isprint(*pTmp)) {
		    trig->pArg = NULL;
		    continue;
		}
		else {
	    	    if (*pTmp == *(trig->inp)) {
			char *inp_buf = pTmp;
			char *trig_buf = trig->inp;

			while (*inp_buf && *trig_buf) {
			    if (*inp_buf != *trig_buf)
				break; 
			    inp_buf++;
			    trig_buf++;
			}

			if (*trig_buf == '\0') {
			    trig->inp = trig_buf;
			    *(trig->pArg++) = '}';
			    *(trig->pArg++) = '\0';	
		    	    trig->pArg = NULL;
			}
			else
			    *(trig->pArg++) = *pTmp;	   
		    }
		    else 
			*(trig->pArg++) = *pTmp;	   
		}
	    }
	    else if (*(trig->inp) == '%') {
		int i = (*(++trig->inp)) - '0';
		
		if (i != 0)
		    break;
		trig->pArg = trig->arg;
		*(trig->pArg++) = '{';
		trig->inp++;
		continue;
	    }
	    else if (*pTmp == *(trig->inp))
		trig->inp++;
	    else {
		trig->inp = trig->in;
	        if (*pTmp == *(trig->inp))
		    trig->inp++;
	    }

	    if (*(trig->inp) == '\0') {
                process_input(trig->out, trig->arg, TRUE);
	        if (trig->level == TRG_LOGIN || trig->level == TRG_PASSWORD)
	            trig->enabled = FALSE;
	        trig->inp = trig->in;
	    }
	    pTmp++;
	}
    }

    return;
}
