/*
 * variable.c for MUDix - Enigma
 */
#include <conf.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <ncurses.h>
#include <panel.h>
#include "mudix.h"


void show_vars(void)
{
    static int		 current;
    	   VAR     	*var;
    	   char    	*str;
    	   int      	 line, i, width, height, centre, wsize;

    width  = LEN_COL-4;
    height = 20;
    wsize  = height-2;
    centre = ((width-2)/2)-1;

    wresize(wMsg, height, width);
    werase(wMsg);
    move_panel(pMsg, 1, 2);

    if (panel_hidden(pMsg) == PANEL_HIDDEN)
	current = 0;

    for (line=1, i=0, var = settings->vars_list; var; var = var->next) {
    	if (!var->name || !var->value)
            continue;

	i+=2;
	if (i < (current*wsize)+2)
	    continue;

      	mvwprintw(wMsg, line++, 1, "Name : %s", var->name);
      	mvwprintw(wMsg, line++, 1, "Value: %s", var->value); 

	if (line >= wsize && var->next) {
	    current++;
	    break;
	}

	if (!var->next)		/* last one */
	    current = 0;
    }

    draw_border(wMsg);
    str = "Variables";
    mvwprintw(wMsg, 0, centre-strlen(str)/2, str);
    show_panel(pMsg);
    return;
}

VAR *new_var(bool last)
{
    VAR *var;

    if (!(var = (VAR *)malloc(sizeof(VAR))))
        return NULL;

    if (last && settings->vars_list) {
        VAR *iVar;

        for (iVar = settings->vars_list; iVar; iVar = iVar->next) {
            if (!iVar->next)
                break;
        }
        iVar->next = var;
        var->next  = NULL;
    }
    else {
        var->next = settings->vars_list;
        settings->vars_list = var;
    }

    var->name  = NULL;
    var->value = NULL;

    return var;
}

void free_var(VAR *var)
{
    VAR *iVar;

    if (!var)
        return;

    for (iVar = settings->vars_list; iVar; iVar = iVar->next) {
        if (iVar->next == var) {
            iVar->next = var->next;
            break;
        }
    }

    if (var == settings->vars_list)
        settings->vars_list = var->next;

    if (var->name)
        free(var->name);
    if (var->value)
        free(var->value);
    free(var);
    return;
}

VAR *var_lookup(char *var)
{
    VAR *iVar;

    for (iVar = settings->vars_list; iVar; iVar = iVar->next) {
	if (!iVar->name || !iVar->value)
	    continue;

    	if (!strcmp(var, iVar->name))
	    break;
    }

    return iVar;
}

bool check_expression(char *expr)
{
    char  cmd1[MAX_STRING], cmd2[MAX_STRING];
    VAR  *var1, *var2;
    bool  equal;
    char *pExpr = expr, *pCmd = cmd1;

    while (*pExpr) {
	if (*pExpr == '!' || *pExpr == '=')
	    break;
	*pCmd++ = *pExpr++;
    }
    *pCmd = '\0';

    if (*pExpr == '\0' || *(pExpr+1) != '=')
	return FALSE;

    equal  = (*pExpr == '=');
    pCmd   = cmd2;
    pExpr += 2;
    while ((*pCmd++ = *pExpr++)) 
	;

    var1 = var_lookup(cmd1);
    var2 = var_lookup(cmd2);

    if (equal)
	return !strcmp(var1? var1->value: cmd1, var2? var2->value: cmd2);
    else
	return strcmp(var1? var1->value: cmd1, var2? var2->value: cmd2);
}

bool process_if(char *buffer)
{
    char str[MAX_STRING];
 
    buffer = get_arg(buffer, str);
    if (!check_expression(str))
	return FALSE;

    get_arg(buffer, str);
    process_input(str, NULL, TRUE);
    return TRUE;    
}
